package com.milieukontakt.reagiram;

import android.app.AlertDialog;
import android.content.Intent;
import android.content.res.Configuration;
import android.os.Bundle;
import android.support.v4.app.ActionBarDrawerToggle;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentTransaction;
import android.support.v4.widget.DrawerLayout;
import android.support.v7.app.ActionBarActivity;
import android.view.MenuItem;
import android.view.View;
import android.view.Window;
import android.widget.AdapterView;
import android.widget.ListView;
import android.widget.TextView;

import com.google.gson.Gson;
import com.milieukontakt.reagiram.adapters.DrawerAdapter;
import com.milieukontakt.reagiram.fragments.AboutFragment;
import com.milieukontakt.reagiram.fragments.ContactFragment;
import com.milieukontakt.reagiram.fragments.HomeFragment;
import com.milieukontakt.reagiram.fragments.StatusesFragment;
import com.milieukontakt.reagiram.models.Profile;
import com.milieukontakt.reagiram.utils.Consts;
import com.milieukontakt.reagiram.utils.Utils;

import java.util.Arrays;
import java.util.List;

public class MainActivity extends ActionBarActivity {

    private ActionBarDrawerToggle mDrawerToggle;
    private DrawerLayout mDrawerLayout;
    private ListView mDrawerList;
    private String mTitle;
    private List<String> mItems;
    private int mCurrentSelection = 1;
    private boolean shouldSwitchFragment = true;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        // Needs to be called before setting the content view
        supportRequestWindowFeature(Window.FEATURE_INDETERMINATE_PROGRESS);
        setContentView(R.layout.activity_main);

        if (Utils.getLongFromPrefs(this, Consts.PREF_KEY_STARTED) == 0) {
            // show initial dialog
            new AlertDialog.Builder(this)
                    .setTitle(R.string.dialog_welcome_title)
                    .setMessage(R.string.dialog_welcome_message)
                    .setPositiveButton(R.string.dialog_welcome_close, null)
                    .show();

            Utils.setLongToPrefs(this, Consts.PREF_KEY_STARTED, 1);
        }
        // initialize the menu drawer
        initDrawer();
    }

    private void initDrawer() {
        mDrawerLayout = (DrawerLayout) findViewById(R.id.drawer_layout);
        mDrawerToggle = new ActionBarDrawerToggle(
                this,                  /* host Activity */
                mDrawerLayout,         /* DrawerLayout object */
                R.drawable.ic_navigation_drawer,  /* nav drawer icon to replace 'Up' caret */
                R.string.drawer_open,  /* "open drawer" description */
                R.string.drawer_close  /* "close drawer" description */
        ) {

            @Override
            public void onDrawerClosed(View drawerView) {
                if (!shouldSwitchFragment)
                    return;
                shouldSwitchFragment = false;
                Fragment fragment = null;
                switch (mCurrentSelection) {
                    case 0:
                        Intent profileActivity = new Intent(MainActivity.this, ProfileActivity.class);
                        startActivity(profileActivity);
                    case 1:
                        fragment = new HomeFragment();
                        break;
                    case 2:
                        fragment = new StatusesFragment();
                        break;
                    case 3:
                        fragment = new ContactFragment();
                        break;
                    case 4:
                        fragment = new AboutFragment();
                        break;
                    default:
                        break;
                }
                if (fragment != null)
                    getSupportFragmentManager()
                            .beginTransaction()
                            .replace(R.id.content_frame, fragment)
                            .setTransition(FragmentTransaction.TRANSIT_ENTER_MASK)
                            .commit();
            }

            @Override
            public void onDrawerOpened(View drawerView) {
                String jsonProfile = Utils.getStringFromPrefs(MainActivity.this, Consts.PREF_KEY_PROFILE);
                if (jsonProfile != null) {
                    Profile profile = new Gson().fromJson(jsonProfile, Profile.class);
                    TextView profileName = (TextView) mDrawerLayout.findViewById(R.id.txt_name);
                    profileName.setText(profile.getFirstName() + " " + profile.getLastName());
                }
                TextView profileReports = (TextView) mDrawerLayout.findViewById(R.id.txt_reports);
                profileReports.setText(String.format(getString(R.string.profile_report_count),
                        Utils.getLongFromPrefs(MainActivity.this, Consts.PREF_KEY_REPORTS_NUMBER)));
            }
        };

        // Set the drawer toggle as the DrawerListener
        mDrawerLayout.setDrawerListener(mDrawerToggle);
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        getSupportActionBar().setHomeButtonEnabled(true);

        mDrawerList = (ListView) findViewById(R.id.left_drawer);

        View drawerHeader = getLayoutInflater().inflate(R.layout.drawer_profile, null);
        // set the list header
        mDrawerList.addHeaderView(drawerHeader);

        // set the list items
        mItems = Arrays.asList(getResources().getStringArray(R.array.drawer_items));
        DrawerAdapter adapter = new DrawerAdapter(this, mItems);
        mDrawerList.setAdapter(adapter);
        mDrawerList.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                selectItem(position);
            }
        });
        // add the home fragment
        mDrawerList.setItemChecked(1, true);
        getSupportFragmentManager()
                .beginTransaction()
                .replace(R.id.content_frame, new HomeFragment())
                .setTransition(FragmentTransaction.TRANSIT_ENTER_MASK)
                .commit();
        setTitle(mItems.get(0));
    }

    @Override
    protected void onPostCreate(Bundle savedInstanceState) {
        super.onPostCreate(savedInstanceState);
        // Sync the toggle state after onRestoreInstanceState has occurred.
        mDrawerToggle.syncState();
    }

    @Override
    public void onConfigurationChanged(Configuration newConfig) {
        super.onConfigurationChanged(newConfig);
        mDrawerToggle.onConfigurationChanged(newConfig);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Pass the event to ActionBarDrawerToggle, if it returns
        // true, then it has handled the app icon touch event
        if (mDrawerToggle.onOptionsItemSelected(item))
            return true;

        return super.onOptionsItemSelected(item);
    }

    // Swaps fragments in the main content view
    private void selectItem(int position) {
        if (position == mCurrentSelection && position != 0) {
            shouldSwitchFragment = false;
            mDrawerLayout.closeDrawer(mDrawerList);
            return;
        }
        shouldSwitchFragment = true;
        mCurrentSelection = position;

        // Highlight the selected item, update the title, and close the drawer
        if (position > 0)
            setTitle(mItems.get(position - 1));
        mDrawerList.setItemChecked(position, true);
        mDrawerLayout.closeDrawer(mDrawerList);
    }

    @Override
    public void setTitle(CharSequence title) {
        mTitle = title.toString();
        getSupportActionBar().setTitle(mTitle);
    }
}
